#!/usr/bin/env python
	
# Authors:Xiaohong Zhuang, Dr.Jeffery B. Klauda
# Date:10/5/2015
	
# This script cut the edge of lipid bilayer
import numpy as np

# Read lipid name
fob = open('lip_lower.txt','r')
lip = fob.readline()
lip = lip.strip()
fob.close()

# Read number of water per lipid
fob = open('num_wat.txt','r')
N_wat = fob.readline()
N_wat = N_wat.strip()
N_wat = int(float(N_wat))
fob.close()

# Read and generate index based on atom names in *sim
filetoread = "%s-atms.txt" % lip
fob=open(filetoread,'r')
atmn_sim=fob.readline() 
atmn_sim=atmn_sim.split()
fob.close()

# Read *.sim for atomic density
filetoread = "%s-nh2c1.sim" % lip
sim_nz=np.genfromtxt(filetoread,delimiter=" ")

# Expected No. of electrons of common atoms. (b(CH1E),m(CH2E),t(CH3E) are united atoms)
# Treat d as h, use d just to have different neutron scattering lengths defined in SIMtoEXP 
N_e={'h':1,'d':1,'c':6,'n':7,'o':8,'p':15,'s':16,'b':7,'m':8,'t':9,'k':18}

for i in range(0,len(atmn_sim),1):
	letters = map(lambda s: s.strip(), atmn_sim[i])
	sim_nz[:,i]=sim_nz[:,i]*N_e[letters[0]]
Tot_Row=np.sum(sim_nz,axis=1)
Dr=Tot_Row.reshape(-1,1)

# Read *.sim for bins
filetoread = "%s-nh2.sim" % lip
sim=np.genfromtxt(filetoread,delimiter="    ")
z=sim[:,0];zr=z.reshape(-1,1)
zD=np.concatenate((zr,Dr),axis=1)

filetowrite = "tot_edp.txt"
np.savetxt(filetowrite,zD,fmt='%-2.5f',delimiter='    ')


# data is an array contains z and density column, bd_s= set boundary slope, w=width of plateau
def plateau(data,dz,bd_s,N_wat):
	if 'pi' in lip:
		w=N_wat/5.0-4.0;
	else:
		w=N_wat/5.0-3.0
	j=int(w/(dz*2)); N=len(data[:,0])-1;
	for i in range(j+1,N/2-1,1):
		dD1=data[i-j+1,1]-data[i-j,1];dD2=data[i+1,1]-data[i,1];dD3=data[i+j+1,1]-data[i+j,1];
		dD4=data[N-i-j+1,1]-data[N-i-j,1];dD5=data[N-i+1,1]-data[N-i,1];dD6=data[N-i+j+1,1]-data[N-i+j,1];
		dDdz1=abs(dD1/dz);dDdz2=abs(dD2/dz);dDdz3=abs(dD3/dz);dDdz4=abs(dD4/dz);dDdz5=abs(dD5/dz);dDdz6=abs(dD6/dz);
		if data[i,1]> 0.3 and abs(dDdz1) < bd_s and dDdz2 < bd_s and dDdz3 < bd_s and dDdz4 < bd_s and dDdz5 < bd_s and dDdz6 < bd_s:
			print 'EDP reach plateau slope %2.7f at %2.7f ' % (dDdz3,data[i,0])
			cut_a = i; cut_b = len(data[:,0]) - cut_a
			return cut_a,cut_b

# (a,b)=plateau(data,dz,bd_s,N_wat)
# May increase bd_s 0.01 up to 0.05 to allow higher plateau fluctuation (up to 1%)
(a,b)=plateau(zD,0.2,0.01,N_wat)

cut_a=min(a,b); cut_b=max(a,b)
sim_ab=sim[cut_a:cut_b,:]

zD_cut=zD[cut_a:cut_b,:]
filetowrite = "tot_edp_cut.txt"
np.savetxt(filetowrite,zD_cut,fmt='%-2.5f',delimiter='    ')

filetowrite = "%s-cut.txt" % lip
np.savetxt(filetowrite,sim_ab,fmt='%-2.5f',delimiter='    ')






