#!/usr/bin/env python
	
# Authors: Xiaohong Zhuang, Dr.Jeffery B. Klauda
# Date: 11/5/2015
	
# This script calculate the thicknesses DHH,DB, and DC
import numpy as np

def calc_thicknesses(lip):
	## Reading and createing the files	
	# Read the group names based on cmp
	filetoread = "%s-nh-grp-names.txt" % lip
	fob=open(filetoread,'r')
	data=fob.readlines()
	gnames = []
	for line in data:
		gnames.append(line)                       # To generate lists of atom names of all lines, with each line as a string element
	gnames = map(lambda s: s.strip(), gnames)        # To remove \n in each string element
	fob.close()

	# Read atom names in cmp and calculate 
	filetoread = "%s-nhc.cmp" % lip
	Ngrp = sum(1 for line in open(filetoread))
	fob=open(filetoread,'r')
	cmp_file=fob.readlines()
	atmn_cmp1 = map(lambda s: s.strip(), cmp_file)
	# Make each string element into list of elements, which make it total list of element list
	for index, line in enumerate(atmn_cmp1):
	    atmn_cmp1[index] = line.split(' ')
	# Take the elments of list out to the main list
	atmn_cmp=[]
	for item in atmn_cmp1:	
		atmn_cmp=atmn_cmp+item
	
	# Read and generate index based on atom names in *sim
	filetoread = "%s-atms.txt" % lip
	fob=open(filetoread,'r')
	atmn_sim=fob.readline() 
	atmn_sim=atmn_sim.split()
	fob.close()

	# Read *.sim for z
	filetoread = "%s.sim" % lip
	simz=np.genfromtxt(filetoread,delimiter=" ",skip_header=2,)
	z=simz[:,0];zr=z.reshape(-1,1)	
	
	# Read *.sim for atomic density
	filetoread = "%s-nh2.sim" % lip
	sim=np.genfromtxt(filetoread,delimiter=" ")
	
	# Import the expected data from E_data.py
	# Expected No. of electrons of common atoms. (b(CH1E),m(CH2E),t(CH3E) are united atoms)
	# Treat d as h, use d just to have different neutron scattering lengths defined in SIMtoEXP 
	N_e={'h':1,'d':1,'c':6,'n':7,'o':8,'p':15,'s':16,'b':7,'m':8,'t':9,'k':18}

	for i in range(0,len(atmn_sim),1):
		letters = map(lambda s: s.strip(), atmn_sim[i])
		sim[:,i]=sim[:,i]*N_e[letters[0]]
	#print np.shape(sim)

	header1 =' Z bin'
	for k in range(0,len(gnames),1):
		header1=header1+'\t '+gnames[k]+' grp'
	header1 = header1 +'\t '+'Comb. C' +'\t '+'Total'

	# Calculate edp per group	
	lineInd=0; 
	for line in cmp_file:
		group = line.strip().split()                          		# To generate a list atoms names of each single line called group
		for i in range(0,len(group),1):
			id_atom_sim = atmn_sim.index(group[i])               	# To find the index of the cmp atom names in atms.txt list
			Ne_sim = sim[:,id_atom_sim].reshape(-1,1);
			if i==0:
				gNe_sim=Ne_sim;
			else:
				gNe_sim=np.concatenate((gNe_sim,Ne_sim),axis=1);
		gNe_rs=np.sum(gNe_sim,axis=1).reshape(-1,1)                                   # axis=0 for column sum; axis=1 for row sum
		if lineInd==0:
			allNe_rs=gNe_rs;
		else:
			allNe_rs=np.concatenate((allNe_rs,gNe_rs),axis=1);
		lineInd = lineInd +1;
	totNe_rs=np.sum(allNe_rs,axis=1).reshape(-1,1);

	# Combine the tail carbon groups
	k=0;
	for j in range(0,len(gnames),1):
		if 'CH' in gnames[j]: # or gnames[j]=='C':
			k=k+1;
			Cgrp=allNe_rs[:,j].reshape(-1,1)	
			if k == 1:
				combineC=Cgrp.reshape(-1,1);
			else:
				combineC=np.concatenate((combineC,Cgrp),axis=1);

	combineC_rs=np.sum(combineC,axis=1).reshape(-1,1); 
	allNe_rs=np.concatenate((allNe_rs,combineC_rs),axis=1);
	allNe_rs=np.concatenate((allNe_rs,totNe_rs),axis=1);
	allNe_rs=np.concatenate((zr,allNe_rs),axis=1);

	filetowrite = "%s-g-edp.dat" % lip
	np.savetxt(filetowrite,allNe_rs,fmt='%1.6f',delimiter='\t',header=header1,)
	
	Nrow=np.shape(allNe_rs)[0]; Nn=Nrow/2;

	# Find DHH
	Tot_mx_n=np.amax(allNe_rs[0:Nn,-1],axis=0);Tot_mx_p=np.amax(allNe_rs[Nn+1:,-1],axis=0);
	In=np.argmax(allNe_rs[0:Nn,-1],axis=0); Ip=np.argmax(allNe_rs[Nn+1:,-1],axis=0);
	DHH_n = allNe_rs[In,0]; DHH_p = allNe_rs[Nn+Ip+1,0]; 
	DHH = DHH_p - DHH_n ; 

	# Find 2DC
	# find V_c at z=0 , take average of -0.1 and 0.1 
	#Rho_max_c=(allNe_rs[Nn-1,-2]+allNe_rs[Nn,-2])/2.0; Rho_mid_c=Rho_max_c/2.0;
	Rho_max_c_n=np.amax(allNe_rs[0:Nn,-2],axis=0); Rho_mid_c_n=Rho_max_c_n/2.0;
	Rho_max_c_p=np.amax(allNe_rs[Nn+1:,-2],axis=0); Rho_mid_c_p=Rho_max_c_p/2.0;
	Rho_max_c=(Rho_max_c_n + Rho_max_c_p)*0.5; Rho_mid_c=(Rho_mid_c_n + Rho_mid_c_p)*0.5;
	#In=np.argmax(allNe_rs[0:Nn,-3],axis=0); Ip=np.argmax(allNe_rs[Nn+1:,-3],axis=0);
	DC_n=np.interp(Rho_mid_c_n, allNe_rs[0:Nn,-2], allNe_rs[0:Nn,0]);
	DC_p=np.interp(Rho_mid_c_p, np.flipud(allNe_rs[Nn+1:,-2]), np.flipud(allNe_rs[Nn+1:,0]));
	DC = DC_p - DC_n; 

	# Find DB
	# find V_w at edge
	Rho_max_w_n=np.amax(allNe_rs[0:Nn,-3],axis=0); Rho_mid_w_n=Rho_max_w_n/2.0;
	Rho_max_w_p=np.amax(allNe_rs[Nn+1:,-3],axis=0); Rho_mid_w_p=Rho_max_w_p/2.0;
	#if 'Pot' in gnames:
		#In=np.argmax(allNe_rs[0:Nn,-3],axis=0); Ip=np.argmax(allNe_rs[Nn+1:,-3],axis=0);
		#V_k=40.7/18.0; Rho_k_n = allNe_rs[In,-4]; Rho_k_p = allNe_rs[Nn+Ip+1,-4];
		# With small trace of K+ present on the edge, the Rho_w is lower. 
		#Rho_mid_w_n = Rho_mid_w_n/(1.0-V_k*Rho_k_n); Rho_mid_w_p = Rho_mid_w_p/(1.0-V_k*Rho_k_p);
	DB_n=np.interp(Rho_mid_w_n, np.flipud(allNe_rs[0:Nn,-3]), np.flipud(allNe_rs[0:Nn,0]));
	DB_p=np.interp(Rho_mid_w_p, allNe_rs[Nn+1:,-3], allNe_rs[Nn+1:,0]);
	DB = DB_p - DB_n; 
	
	# print the results
	print '%s\tDHH\t%2.1f\t%2.1f\t%2.1f\t%2.4f\t%2.4f' % (lip.upper(),DHH,DHH_p,DHH_n,Tot_mx_n,Tot_mx_p)
	print '%s\tDB\t%2.1f\t%2.1f\t%2.1f\t%2.4f\t%2.4f' % (lip.upper(),DB,DB_p,DB_n,Rho_max_w_n,Rho_mid_w_n)
	print '%s\t2DC\t%2.1f\t%2.1f\t%2.1f\t%2.4f\t%2.4f' % (lip.upper(),DC,DC_p,DC_n,Rho_max_c,Rho_mid_c)

